@@ -48,6 +48,7 @@ static gboolean notification_plugin_size_changed (XfcePanelPlugin *plugin,
                                                   gint size,
                                                   NotificationPlugin *notification_plugin);
 
+static void notification_plugin_init_log_proxy(NotificationPlugin *notification_plugin);
 
 GtkWidget *
 notification_plugin_menu_new (NotificationPlugin *notification_plugin)
@@ -165,6 +166,7 @@ cb_hide_on_read_changed(XfconfChannel *channel,
 void
 notification_plugin_update_icon(NotificationPlugin *notification_plugin) {
   GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
+  GtkStyleContext *style_context = gtk_widget_get_style_context(notification_plugin->image);
   GIcon *base_icon;
   GtkIconInfo *icon_info;
   gint scale_factor;
@@ -187,7 +189,7 @@ notification_plugin_update_icon(NotificationPlugin *notification_plugin) {
                                                        GTK_ICON_LOOKUP_FORCE_SIZE);
 
   if (G_LIKELY(icon_info != NULL)) {
-    GdkPixbuf *pix = gtk_icon_info_load_icon(icon_info, NULL);
+    GdkPixbuf *pix = gtk_icon_info_load_symbolic_for_context(icon_info, style_context, NULL, NULL);
 
     if (G_LIKELY(pix != NULL)) {
       cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf(pix, scale_factor, NULL);
@@ -201,7 +203,6 @@ notification_plugin_update_icon(NotificationPlugin *notification_plugin) {
                                                                             GTK_ICON_LOOKUP_FORCE_SIZE);
 
         if (G_LIKELY(emblem_info != NULL)) {
-          GtkStyleContext *style_context = gtk_widget_get_style_context(notification_plugin->button);
           GdkPixbuf *emblem_pix = gtk_icon_info_load_symbolic_for_context(emblem_info, style_context, NULL, NULL);
 
           if (G_LIKELY(emblem_pix != NULL)) {
@@ -253,11 +254,102 @@ notification_plugin_dnd_updated (XfconfChannel *channel,
 
 
 static void
-cb_notification_log_changed(XfceNotifyLog *log,
-                            NotificationPlugin *notification_plugin)
+notification_plugin_has_unread_ready(GObject *source,
+                                     GAsyncResult *res,
+                                     NotificationPlugin *notification_plugin)
 {
-  notification_plugin->new_notifications = xfce_notify_log_has_unread_messages(notification_plugin->log);
-  notification_plugin_update_icon(notification_plugin);
+    gboolean success;
+    gboolean has_unread = FALSE;
+    GError *error = NULL;
+
+    success = xfce_notify_log_gbus_call_has_unread_finish(XFCE_NOTIFY_LOG_GBUS(source),
+                                                          &has_unread,
+                                                          res,
+                                                          &error);
+    if (G_UNLIKELY(!success)) {
+        g_warning("Unable to check for unread messages: %s", error != NULL ? error->message : "(unknown)");
+        if (error != NULL) {
+            g_error_free(error);
+        }
+    }
+
+    notification_plugin->new_notifications = has_unread;
+    notification_plugin_update_icon(notification_plugin);
+}
+
+
+
+static void
+notification_plugin_log_changed(NotificationPlugin *notification_plugin) {
+  xfce_notify_log_gbus_call_has_unread(notification_plugin->log,
+                                       NULL,
+                                       (GAsyncReadyCallback)notification_plugin_has_unread_ready,
+                                       notification_plugin);
+}
+
+
+
+static gboolean
+notification_plugin_connect_log_proxy(NotificationPlugin *notification_plugin) {
+  notification_plugin->log_proxy_connect_id = 0;
+  notification_plugin_init_log_proxy(notification_plugin);
+  return G_SOURCE_REMOVE;
+}
+
+
+
+static void
+notification_plugin_bus_proxy_connected(GObject *source,
+                                        GAsyncResult *res,
+                                        NotificationPlugin *notification_plugin)
+{
+  GError *error = NULL;
+
+  notification_plugin->log = xfce_notify_log_gbus_proxy_new_finish(res, &error);
+  if (G_UNLIKELY(notification_plugin->log == NULL)) {
+    g_warning("Could not connect to notification daemon; log will be unavailable: %s", error != NULL ? error->message : "(unknown)");
+    if (error != NULL) {
+      g_error_free(error);
+    }
+
+    if (notification_plugin->log_proxy_connect_id == 0) {
+      notification_plugin->log_proxy_connect_id = g_timeout_add_seconds(1,
+                                                                        (GSourceFunc)notification_plugin_connect_log_proxy,
+                                                                        notification_plugin);
+    }
+  } else {
+    g_dbus_proxy_set_default_timeout(G_DBUS_PROXY(notification_plugin->log), 1500);
+
+    g_signal_connect_swapped(notification_plugin->log, "row-added",
+                             G_CALLBACK(notification_plugin_log_changed), notification_plugin);
+    g_signal_connect_swapped(notification_plugin->log, "row-changed",
+                             G_CALLBACK(notification_plugin_log_changed), notification_plugin);
+    g_signal_connect_swapped(notification_plugin->log, "row-deleted",
+                             G_CALLBACK(notification_plugin_log_changed), notification_plugin);
+    g_signal_connect_swapped(notification_plugin->log, "truncated",
+                             G_CALLBACK(notification_plugin_log_changed), notification_plugin);
+    g_signal_connect_swapped(notification_plugin->log, "cleared",
+                             G_CALLBACK(notification_plugin_log_changed), notification_plugin);
+    xfce_notify_log_gbus_call_has_unread(notification_plugin->log,
+                                         NULL,
+                                         (GAsyncReadyCallback)notification_plugin_has_unread_ready,
+                                         notification_plugin);
+  }
+}
+
+
+
+static void
+notification_plugin_init_log_proxy(NotificationPlugin *notification_plugin) {
+  if (notification_plugin->log == NULL) {
+    xfce_notify_log_gbus_proxy_new_for_bus(G_BUS_TYPE_SESSION,
+                                           0,
+                                           "org.xfce.Notifyd",
+                                           "/org/xfce/Notifyd",
+                                           NULL,
+                                           (GAsyncReadyCallback)notification_plugin_bus_proxy_connected,
+                                           notification_plugin);
+  }
 }
 
 
@@ -266,23 +358,11 @@ static NotificationPlugin *
 notification_plugin_new (XfcePanelPlugin *panel_plugin)
 {
   NotificationPlugin    *notification_plugin;
-  GError *error = NULL;
 
   /* Allocate memory for the plugin structure */
   notification_plugin = g_slice_new0 (NotificationPlugin);
   notification_plugin->plugin = panel_plugin;
 
-  notification_plugin->log = xfce_notify_log_open(&error);
-  if (notification_plugin->log == NULL) {
-    notification_plugin->new_notifications = FALSE;
-    g_critical("Unable to open notification log: %s", error->message);
-    g_error_free(error);
-  } else {
-    notification_plugin->new_notifications = xfce_notify_log_has_unread_messages(notification_plugin->log);
-    g_signal_connect(notification_plugin->log, "changed",
-                     G_CALLBACK(cb_notification_log_changed), notification_plugin);
-  }
-
   /* Initialize xfconf */
   xfconf_init (NULL);
   notification_plugin->channel = xfconf_channel_new ("xfce4-notifyd");
@@ -317,11 +397,15 @@ notification_plugin_new (XfcePanelPlugin *panel_plugin)
                     G_CALLBACK (cb_menu_deactivate), notification_plugin);
   g_signal_connect (notification_plugin->menu, "size-allocate",
                     G_CALLBACK (cb_menu_size_allocate), notification_plugin);
+  g_signal_connect_swapped(gtk_icon_theme_get_default(), "changed",
+                           G_CALLBACK(notification_plugin_update_icon), notification_plugin);
 
   /* Start monitoring the "do not disturb" setting in xfconf */
   g_signal_connect (G_OBJECT (notification_plugin->channel), "property-changed::" "/do-not-disturb",
                     G_CALLBACK (notification_plugin_dnd_updated), notification_plugin);
 
+  notification_plugin_init_log_proxy(notification_plugin);
+
   return notification_plugin;
 }
 
@@ -333,10 +417,18 @@ notification_plugin_free (XfcePanelPlugin *plugin,
 {
   GtkWidget *dialog;
 
-  if (notification_plugin->log) {
-      g_object_unref(notification_plugin->log);
+  if (notification_plugin->log_proxy_connect_id != 0) {
+    g_source_remove(notification_plugin->log_proxy_connect_id);
   }
 
+  if (notification_plugin->log != NULL) {
+    g_object_unref(notification_plugin->log);
+  }
+
+  g_signal_handlers_disconnect_by_func(gtk_icon_theme_get_default(),
+                                       notification_plugin_update_icon,
+                                       notification_plugin);
+
   /* check if the dialog is still open. if so, destroy it */
   dialog = g_object_get_data (G_OBJECT (plugin), "dialog");
   if (G_UNLIKELY (dialog != NULL))
