@@ -44,11 +44,14 @@
 #endif
 
 #include <common/xfce-notify-common.h>
-#include <common/xfce-notify-log.h>
+#include <common/xfce-notify-log-gbus.h>
 #include <common/xfce-notify-log-util.h>
 
-#include "xfce-notify-gbus.h"
 #include "xfce-notify-daemon.h"
+#include "xfce-notify-daemon-log.h"
+#include "xfce-notify-fdo-gbus.h"
+#include "xfce-notify-gbus.h"
+#include "xfce-notify-log.h"
 #include "xfce-notify-window.h"
 #include "xfce-notify-marshal.h"
 
@@ -57,13 +60,16 @@
 
 struct _XfceNotifyDaemon
 {
-    XfceNotifyGBusSkeleton parent;
+    XfceNotifyFdoGBusSkeleton parent;
+
+    XfceNotifyGBus *xfce_iface_skeleton;
+    XfceNotifyOldGBus *xfce_old_iface_skeleton;
 
-    XfceNotifyOrgXfceNotifyd *xfce_iface_skeleton;
     gboolean expire_timeout_enabled;
     gint expire_timeout;
     gboolean expire_timeout_allow_override;
     guint bus_name_id;
+    guint notifyd_bus_name_id;
     gdouble initial_opacity;
     GtkCornerType notify_location;
     gboolean do_fadeout;
@@ -72,7 +78,9 @@ struct _XfceNotifyDaemon
     gboolean notification_log;
     gboolean show_text_with_gauge;
     gint primary_monitor;
+    gboolean windows_use_override_redirect;
 
+    XfceNotifyDaemonLog *xndlog;
     XfceNotifyLog *log;
     gint log_level;
     gint log_level_apps;
@@ -100,8 +108,7 @@ struct _XfceNotifyDaemon
 
 typedef struct
 {
-    XfceNotifyGBusSkeletonClass  parent;
-
+    XfceNotifyFdoGBusSkeletonClass parent_class;
 } XfceNotifyDaemonClass;
 
 
@@ -223,6 +230,12 @@ const struct {
         .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, log_max_size),
         .default_value.u = LOG_MAX_SIZE_DEFAULT,
     },
+    {
+        .name = COMPAT_OVERRIDE_REDIRECT_PROP,
+        .type = G_TYPE_BOOLEAN,
+        .offset = G_STRUCT_OFFSET(XfceNotifyDaemon, windows_use_override_redirect),
+        .default_value.b = FALSE,
+    },
 };
 
 static void xfce_notify_daemon_screen_changed(GdkScreen *screen,
@@ -231,7 +244,7 @@ static gboolean xfce_notify_daemon_update_reserved_rectangles(gpointer key,
                                                               gpointer value,
                                                               gpointer data);
 static void xfce_notify_daemon_finalize(GObject *obj);
-static void  xfce_notify_daemon_constructed(GObject *obj);
+static void xfce_notify_daemon_constructed(GObject *obj);
 
 static GQuark xfce_notify_daemon_get_n_monitors_quark(void);
 
@@ -244,9 +257,9 @@ static GdkFilterReturn xfce_notify_rootwin_watch_workarea(GdkXEvent *gxevent,
 static void daemon_quit (XfceNotifyDaemon *xndaemon);
 
 /* DBus method callbacks  forward declarations */
-static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
-                                		 GDBusMethodInvocation   *invocation,
-                                         XfceNotifyDaemon *xndaemon);
+static gboolean notify_get_capabilities(XfceNotifyFdoGBus *skeleton,
+                                        GDBusMethodInvocation   *invocation,
+                                        XfceNotifyDaemon *xndaemon);
 
 static void notify_update_known_applications (XfconfChannel *channel,
                                               gchar *app_name);
@@ -254,36 +267,40 @@ static void notify_update_known_applications (XfconfChannel *channel,
 static gboolean notify_application_is_muted (XfconfChannel *channel,
                                              gchar *new_app_name);
 
-static gboolean notify_notify (XfceNotifyGBus *skeleton,
-                               GDBusMethodInvocation   *invocation,
-                               const gchar *app_name,
-                               guint replaces_id,
-                               const gchar *app_icon,
-                               const gchar *summary,
-                               const gchar *body,
-                               const gchar **actions,
-                               GVariant *hints,
-                               gint expire_timeout,
-                               XfceNotifyDaemon *xndaemon);
+static gboolean notify_notify(XfceNotifyFdoGBus *skeleton,
+                              GDBusMethodInvocation   *invocation,
+                              const gchar *app_name,
+                              guint replaces_id,
+                              const gchar *app_icon,
+                              const gchar *summary,
+                              const gchar *body,
+                              const gchar **actions,
+                              GVariant *hints,
+                              gint expire_timeout,
+                              XfceNotifyDaemon *xndaemon);
 
 
-static gboolean notify_close_notification (XfceNotifyGBus *skeleton,
-                                           GDBusMethodInvocation   *invocation,
-                                           guint id,
-                                           XfceNotifyDaemon *xndaemon);
+static gboolean notify_close_notification(XfceNotifyFdoGBus *skeleton,
+                                          GDBusMethodInvocation *invocation,
+                                          guint id,
+                                          XfceNotifyDaemon *xndaemon);
 
 
-static gboolean notify_get_server_information (XfceNotifyGBus *skeleton,
-                                               GDBusMethodInvocation *invocation,
-                                               XfceNotifyDaemon *xndaemon);
+static gboolean notify_get_server_information(XfceNotifyFdoGBus *skeleton,
+                                              GDBusMethodInvocation *invocation,
+                                              XfceNotifyDaemon *xndaemon);
 
 
-static gboolean notify_quit (XfceNotifyOrgXfceNotifyd *skeleton,
-                             GDBusMethodInvocation   *invocation,
-                             XfceNotifyDaemon *xndaemon);
+static gboolean xfce_notify_quit(XfceNotifyGBus *skeleton,
+                                 GDBusMethodInvocation *invocation,
+                                 XfceNotifyDaemon *xndaemon);
+
+static gboolean xfce_notify_old_quit(XfceNotifyOldGBus *skeleton,
+                                     GDBusMethodInvocation *invocation,
+                                     XfceNotifyDaemon *xndaemon);
 
 
-G_DEFINE_TYPE(XfceNotifyDaemon, xfce_notify_daemon, XFCE_NOTIFY_TYPE_GBUS_SKELETON)
+G_DEFINE_TYPE(XfceNotifyDaemon, xfce_notify_daemon, XFCE_TYPE_NOTIFY_FDO_GBUS_SKELETON)
 
 
 static void
@@ -442,7 +459,6 @@ xfce_notify_daemon_init_placement_data(XfceNotifyDaemon *xndaemon)
 #endif
 }
 
-
 static void
 xfce_notify_bus_name_acquired_cb (GDBusConnection *connection,
                                   const gchar *name,
@@ -452,48 +468,66 @@ xfce_notify_bus_name_acquired_cb (GDBusConnection *connection,
     GError *error = NULL;
     gboolean exported;
 
-
     xndaemon = XFCE_NOTIFY_DAEMON(user_data);
 
-    exported =  g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (xndaemon),
-                                                  connection,
-                                                  "/org/freedesktop/Notifications",
-                                                  &error);
-    if (exported)
-    {
-        /* Connect dbus signals callbacks */
-        g_signal_connect (xndaemon, "handle-notify",
-                          G_CALLBACK(notify_notify), xndaemon);
+    if (g_strcmp0(name, "org.freedesktop.Notifications") == 0) {
+        exported =  g_dbus_interface_skeleton_export(G_DBUS_INTERFACE_SKELETON (xndaemon),
+                                                     connection,
+                                                     "/org/freedesktop/Notifications",
+                                                     &error);
+        if (exported) {
+            /* Connect dbus signals callbacks */
+            g_signal_connect(xndaemon, "handle-notify",
+                             G_CALLBACK(notify_notify), xndaemon);
 
-        g_signal_connect (xndaemon, "handle-get-capabilities",
-                          G_CALLBACK(notify_get_capabilities), xndaemon);
+            g_signal_connect(xndaemon, "handle-get-capabilities",
+                             G_CALLBACK(notify_get_capabilities), xndaemon);
 
-        g_signal_connect (xndaemon, "handle-get-server-information",
-                          G_CALLBACK(notify_get_server_information), xndaemon);
+            g_signal_connect(xndaemon, "handle-get-server-information",
+                             G_CALLBACK(notify_get_server_information), xndaemon);
 
-        g_signal_connect (xndaemon, "handle-close-notification",
-                          G_CALLBACK(notify_close_notification), xndaemon);
-    }
-    else
-    {
-        g_warning ("Failed to export interface: %s", error->message);
-        g_error_free (error);
-        gtk_main_quit ();
-    }
+            g_signal_connect(xndaemon, "handle-close-notification",
+                             G_CALLBACK(notify_close_notification), xndaemon);
+        } else {
+            g_warning("Failed to export interface: %s", error->message);
+            g_error_free(error);
+            gtk_main_quit();
+        }
 
-    xndaemon->xfce_iface_skeleton  = xfce_notify_org_xfce_notifyd_skeleton_new();
-    exported =  g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_iface_skeleton),
-                                                  connection,
-                                                  "/org/freedesktop/Notifications",
-                                                  &error);
-    if (exported)
-        g_signal_connect (xndaemon->xfce_iface_skeleton, "handle-quit",
-                          G_CALLBACK(notify_quit), xndaemon);
-    else
-    {
-        g_warning ("Failed to export interface: %s", error->message);
-        g_error_free (error);
-        gtk_main_quit ();
+        xndaemon->xfce_old_iface_skeleton  = xfce_notify_old_gbus_skeleton_new();
+        exported = g_dbus_interface_skeleton_export(G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_old_iface_skeleton),
+                                                    connection,
+                                                    "/org/freedesktop/Notifications",
+                                                    &error);
+        if (exported) {
+            g_signal_connect(xndaemon->xfce_old_iface_skeleton, "handle-quit",
+                             G_CALLBACK(xfce_notify_old_quit), xndaemon);
+        } else {
+            g_warning("Failed to export interface: %s", error->message);
+            g_error_free(error);
+            gtk_main_quit();
+        }
+    } else if (g_strcmp0(name, "org.xfce.Notifyd") == 0) {
+        xndaemon->xfce_iface_skeleton  = xfce_notify_gbus_skeleton_new();
+        exported = g_dbus_interface_skeleton_export(G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_iface_skeleton),
+                                                    connection,
+                                                    "/org/xfce/Notifyd",
+                                                    &error);
+        if (exported) {
+            g_signal_connect(xndaemon->xfce_iface_skeleton, "handle-quit",
+                             G_CALLBACK(xfce_notify_quit), xndaemon);
+        } else {
+            g_warning("Failed to export interface: %s", error->message);
+            g_error_free(error);
+            gtk_main_quit();
+        }
+
+        xndaemon->xndlog = xfce_notify_daemon_log_new(connection, xndaemon->log, &error);
+        if (xndaemon->xndlog == NULL) {
+            g_warning("Failed to export interface: %s", error->message);
+            g_error_free(error);
+            gtk_main_quit();
+        }
     }
 }
 
@@ -506,11 +540,9 @@ xfce_notify_bus_name_lost_cb (GDBusConnection *connection,
     daemon_quit(XFCE_NOTIFY_DAEMON(user_data));
 }
 
-static void xfce_notify_daemon_constructed (GObject *obj)
-{
-    XfceNotifyDaemon *self;
-
-    self  = XFCE_NOTIFY_DAEMON (obj);
+static void
+xfce_notify_daemon_constructed(GObject *obj) {
+    XfceNotifyDaemon *self = XFCE_NOTIFY_DAEMON (obj);
 
     self->bus_name_id = g_bus_own_name (G_BUS_TYPE_SESSION,
                                         "org.freedesktop.Notifications",
@@ -520,6 +552,49 @@ static void xfce_notify_daemon_constructed (GObject *obj)
                                         xfce_notify_bus_name_lost_cb,
                                         self,
                                         NULL);
+    self->notifyd_bus_name_id = g_bus_own_name(G_BUS_TYPE_SESSION,
+                                               "org.xfce.Notifyd",
+                                               G_BUS_NAME_OWNER_FLAGS_REPLACE,
+                                               xfce_notify_bus_name_acquired_cb,
+                                               NULL,
+                                               xfce_notify_bus_name_lost_cb,
+                                               self,
+                                               NULL);
+}
+
+static void
+log_row_added(XfceNotifyLog *log, const gchar *entry_id, XfceNotifyDaemon *xndaemon) {
+    if (xndaemon->xndlog != NULL) {
+        xfce_notify_log_gbus_emit_row_added(XFCE_NOTIFY_LOG_GBUS(xndaemon->xndlog), entry_id);
+    }
+}
+
+static void
+log_row_changed(XfceNotifyLog *log, const gchar *entry_id, XfceNotifyDaemon *xndaemon) {
+    if (xndaemon->xndlog != NULL) {
+        xfce_notify_log_gbus_emit_row_changed(XFCE_NOTIFY_LOG_GBUS(xndaemon->xndlog), entry_id != NULL ? entry_id : "");
+    }
+}
+
+static void
+log_row_deleted(XfceNotifyLog *log, const gchar *entry_id, XfceNotifyDaemon *xndaemon) {
+    if (xndaemon->xndlog != NULL) {
+        xfce_notify_log_gbus_emit_row_deleted(XFCE_NOTIFY_LOG_GBUS(xndaemon->xndlog), entry_id);
+    }
+}
+
+static void
+log_truncated(XfceNotifyLog *log, guint n_kept_entries, XfceNotifyDaemon *xndaemon) {
+    if (xndaemon->xndlog != NULL) {
+        xfce_notify_log_gbus_emit_truncated(XFCE_NOTIFY_LOG_GBUS(xndaemon->xndlog), n_kept_entries);
+    }
+}
+
+static void
+log_cleared(XfceNotifyLog *log, XfceNotifyDaemon *xndaemon) {
+    if (xndaemon->xndlog != NULL) {
+        xfce_notify_log_gbus_emit_cleared(XFCE_NOTIFY_LOG_GBUS(xndaemon->xndlog));
+    }
 }
 
 static void
@@ -547,6 +622,17 @@ xfce_notify_daemon_init(XfceNotifyDaemon *xndaemon)
         if (error != NULL) {
             g_error_free(error);
         }
+    } else {
+        g_signal_connect(xndaemon->log, "row-added",
+                         G_CALLBACK(log_row_added), xndaemon);
+        g_signal_connect(xndaemon->log, "row-changed",
+                         G_CALLBACK(log_row_changed), xndaemon);
+        g_signal_connect(xndaemon->log, "row-deleted",
+                         G_CALLBACK(log_row_deleted), xndaemon);
+        g_signal_connect(xndaemon->log, "truncated",
+                         G_CALLBACK(log_truncated), xndaemon);
+        g_signal_connect(xndaemon->log, "cleared",
+                         G_CALLBACK(log_cleared), xndaemon);
     }
 }
 
@@ -565,13 +651,16 @@ xfce_notify_daemon_finalize(GObject *obj)
         g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON(xndaemon));
     }
 
-    if (xndaemon->xfce_iface_skeleton &&
-        g_dbus_interface_skeleton_has_connection(G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_iface_skeleton),
+    if (xndaemon->xfce_old_iface_skeleton &&
+        g_dbus_interface_skeleton_has_connection(G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_old_iface_skeleton),
                                                  connection))
     {
-    g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_iface_skeleton));
+        g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON(xndaemon->xfce_old_iface_skeleton));
     }
 
+    if (xndaemon->xndlog != NULL) {
+        g_object_unref(xndaemon->xndlog);
+    }
 
     if(xndaemon->reserved_rectangles && xndaemon->monitors_workarea) {
       GdkScreen *screen = gdk_screen_get_default ();
@@ -631,9 +720,9 @@ xfce_notify_daemon_window_action_invoked(XfceNotifyWindow *window,
     guint id = xfce_notify_window_get_id(window);
 
     if (G_LIKELY(id > 0)) {
-        xfce_notify_gbus_emit_action_invoked (XFCE_NOTIFY_GBUS(xndaemon),
-                                              id,
-                                              action);
+        xfce_notify_fdo_gbus_emit_action_invoked(XFCE_NOTIFY_FDO_GBUS(xndaemon),
+                                                 id,
+                                                 action);
     } else {
         g_warning("Notify window somehow didn't have an ID");
     }
@@ -665,9 +754,9 @@ xfce_notify_daemon_window_closed(XfceNotifyWindow *window,
     if (G_LIKELY(id > 0)) {
         g_tree_remove(xndaemon->active_notifications, GUINT_TO_POINTER(id));
 
-        xfce_notify_gbus_emit_notification_closed (XFCE_NOTIFY_GBUS(xndaemon),
-                                                   id,
-                                                   (guint)reason);
+        xfce_notify_fdo_gbus_emit_notification_closed(XFCE_NOTIFY_FDO_GBUS(xndaemon),
+                                                      id,
+                                                      (guint)reason);
     } else {
         g_warning("Notify window somehow didn't have an ID");
     }
@@ -933,9 +1022,9 @@ xfce_notify_daemon_update_reserved_rectangles(gpointer key,
 
 
 
-static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
-                                         GDBusMethodInvocation   *invocation,
-                                         XfceNotifyDaemon *xndaemon)
+static gboolean notify_get_capabilities(XfceNotifyFdoGBus *skeleton,
+                                        GDBusMethodInvocation *invocation,
+                                        XfceNotifyDaemon *xndaemon)
 {
     const gchar *const capabilities[] = {
         "action-icons",
@@ -951,7 +1040,7 @@ static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
         NULL,
     };
 
-    xfce_notify_gbus_complete_get_capabilities(skeleton, invocation, capabilities);
+    xfce_notify_fdo_gbus_complete_get_capabilities(skeleton, invocation, capabilities);
 
     return TRUE;
 }
@@ -1148,17 +1237,17 @@ xfce_notify_log_insert(XfceNotifyLog *log,
 }
 
 static gboolean
-notify_notify (XfceNotifyGBus *skeleton,
-               GDBusMethodInvocation   *invocation,
-               const gchar *app_name,
-               guint replaces_id,
-               const gchar *app_icon,
-               const gchar *summary,
-               const gchar *body,
-               const gchar **actions,
-               GVariant *hints,
-               gint expire_timeout,
-               XfceNotifyDaemon *xndaemon)
+notify_notify(XfceNotifyFdoGBus *skeleton,
+              GDBusMethodInvocation   *invocation,
+              const gchar *app_name,
+              guint replaces_id,
+              const gchar *app_icon,
+              const gchar *summary,
+              const gchar *body,
+              const gchar **actions,
+              GVariant *hints,
+              gint expire_timeout,
+              XfceNotifyDaemon *xndaemon)
 {
     XfceNotifyWindow *window;
     GDateTime *timestamp;
@@ -1368,7 +1457,7 @@ notify_notify (XfceNotifyGBus *skeleton,
                       }
             }
 
-            xfce_notify_gbus_complete_notify (skeleton, invocation, OUT_id);
+            xfce_notify_fdo_gbus_complete_notify (skeleton, invocation, OUT_id);
             if (image_data)
                 g_variant_unref (image_data);
             if (desktop_id)
@@ -1403,6 +1492,7 @@ notify_notify (XfceNotifyGBus *skeleton,
                                                                         actions_are_icon_names,
                                                                         xndaemon->css_provider));
         xfce_notify_window_set_id(window, OUT_id);
+        xfce_notify_window_set_override_redirect(window, xndaemon->windows_use_override_redirect);
         xfce_notify_window_set_opacity(window, xndaemon->initial_opacity);
 #ifdef ENABLE_SOUND
         xfce_notify_window_set_sound_props(window, sound_props);
@@ -1494,7 +1584,7 @@ notify_notify (XfceNotifyGBus *skeleton,
 
     gtk_widget_realize(GTK_WIDGET(window));
 
-    xfce_notify_gbus_complete_notify(skeleton, invocation, OUT_id);
+    xfce_notify_fdo_gbus_complete_notify(skeleton, invocation, OUT_id);
 
     if (image_data)
       g_variant_unref (image_data);
@@ -1508,10 +1598,11 @@ notify_notify (XfceNotifyGBus *skeleton,
 }
 
 
-static gboolean notify_close_notification (XfceNotifyGBus *skeleton,
-                                           GDBusMethodInvocation   *invocation,
-                                           guint id,
-                                           XfceNotifyDaemon *xndaemon)
+static gboolean
+notify_close_notification(XfceNotifyFdoGBus *skeleton,
+                          GDBusMethodInvocation *invocation,
+                          guint id,
+                          XfceNotifyDaemon *xndaemon)
 {
     XfceNotifyWindow *window = g_tree_lookup(xndaemon->active_notifications,
                                              GUINT_TO_POINTER(id));
@@ -1519,39 +1610,49 @@ static gboolean notify_close_notification (XfceNotifyGBus *skeleton,
     if(window)
         xfce_notify_window_closed(window, XFCE_NOTIFY_CLOSE_REASON_CLIENT);
 
-    xfce_notify_gbus_complete_close_notification(skeleton, invocation);
+    xfce_notify_fdo_gbus_complete_close_notification(skeleton, invocation);
 
     return TRUE;
 }
 
-static gboolean notify_get_server_information (XfceNotifyGBus *skeleton,
-                                               GDBusMethodInvocation   *invocation,
-                                               XfceNotifyDaemon *xndaemon)
+static gboolean
+notify_get_server_information (XfceNotifyFdoGBus *skeleton,
+                               GDBusMethodInvocation *invocation,
+                               XfceNotifyDaemon *xndaemon)
 {
-    xfce_notify_gbus_complete_get_server_information(skeleton,
-                                                     invocation,
-                                                     "Xfce Notify Daemon",
-                                                     "Xfce",
-                                                     VERSION,
-                                                     NOTIFICATIONS_SPEC_VERSION);
+    xfce_notify_fdo_gbus_complete_get_server_information(skeleton,
+                                                         invocation,
+                                                         "Xfce Notify Daemon",
+                                                         "Xfce",
+                                                         VERSION,
+                                                         NOTIFICATIONS_SPEC_VERSION);
 
     return TRUE;
 }
 
 
-static void daemon_quit (XfceNotifyDaemon *xndaemon)
-{
+static void
+daemon_quit(XfceNotifyDaemon *xndaemon) {
     gint i, main_level = gtk_main_level();
     for(i = 0; i < main_level; ++i)
         gtk_main_quit();
 }
 
 
-static gboolean notify_quit (XfceNotifyOrgXfceNotifyd *skeleton,
-                             GDBusMethodInvocation   *invocation,
-                             XfceNotifyDaemon *xndaemon)
-{
-    xfce_notify_org_xfce_notifyd_complete_quit (skeleton, invocation);
+static gboolean
+xfce_notify_old_quit(XfceNotifyOldGBus *skeleton, GDBusMethodInvocation *invocation, XfceNotifyDaemon *xndaemon) {
+    g_message("Method org.freedesktop.Notifications.Quit is deprecated and will be removed in a future version; please use org.xfce.Notifyd.Quit instead");
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    xfce_notify_old_gbus_complete_quit(skeleton, invocation);
+G_GNUC_END_IGNORE_DEPRECATIONS
+    daemon_quit(xndaemon);
+    return TRUE;
+}
+
+
+static gboolean
+xfce_notify_quit(XfceNotifyGBus *skeleton, GDBusMethodInvocation *invocation, XfceNotifyDaemon *xndaemon) {
+    xfce_notify_gbus_complete_quit(skeleton, invocation);
     daemon_quit(xndaemon);
     return TRUE;
 }
