@@ -43,7 +43,6 @@
 #include <common/xfce-notify-common.h>
 
 #include "xfce-notify-window.h"
-#include "xfce-notify-enum-types.h"
 
 #define DEFAULT_EXPIRE_TIMEOUT 10000
 #define DEFAULT_NORMAL_OPACITY 0.85
@@ -65,6 +64,7 @@ struct _XfceNotifyWindow
 
     GdkRectangle geometry;
     gint last_monitor;
+    gboolean override_redirect;
 
     guint expire_timeout;
 
@@ -140,6 +140,24 @@ static void xfce_notify_window_button_clicked(GtkWidget *widget,
 
 static guint signals[N_SIGS] = { 0, };
 
+GType
+xfce_notify_close_reason_get_type(void)
+{
+	static GType type = 0;
+
+    if (type == 0) {
+        static const GEnumValue values[] = {
+            { XFCE_NOTIFY_CLOSE_REASON_EXPIRED, "XFCE_NOTIFY_CLOSE_REASON_EXPIRED", "expired" },
+            { XFCE_NOTIFY_CLOSE_REASON_DISMISSED, "XFCE_NOTIFY_CLOSE_REASON_DISMISSED", "dismissed" },
+            { XFCE_NOTIFY_CLOSE_REASON_CLIENT, "XFCE_NOTIFY_CLOSE_REASON_CLIENT", "client" },
+            { XFCE_NOTIFY_CLOSE_REASON_UNKNOWN, "XFCE_NOTIFY_CLOSE_REASON_UNKNOWN", "unknown" },
+            { 0, NULL, NULL },
+        };
+        type = g_enum_register_static("XfceNotifyCloseReason", values);
+    }
+
+    return type;
+}
 
 G_DEFINE_TYPE(XfceNotifyWindow, xfce_notify_window, GTK_TYPE_WINDOW)
 
@@ -380,6 +398,8 @@ xfce_notify_window_realize(GtkWidget *widget)
 
     GTK_WIDGET_CLASS(xfce_notify_window_parent_class)->realize(widget);
 
+    gdk_window_set_override_redirect(gtk_widget_get_window(widget), window->override_redirect);
+
     xfce_notify_window_start_expiration(window);
 
 #ifdef ENABLE_SOUND
@@ -1210,6 +1230,22 @@ xfce_notify_window_set_sound_props(XfceNotifyWindow *window,
 }
 #endif
 
+void
+xfce_notify_window_set_override_redirect(XfceNotifyWindow *window,
+                                         gboolean override_redirect)
+{
+    g_return_if_fail(XFCE_IS_NOTIFY_WINDOW(window));
+
+    if (window->override_redirect != override_redirect) {
+        window->override_redirect = override_redirect;
+
+        if (gtk_widget_get_realized(GTK_WIDGET(window))) {
+            gdk_window_set_override_redirect(gtk_widget_get_window(GTK_WIDGET(window)),
+                                             window->override_redirect);
+        }
+    }
+}
+
 void
 xfce_notify_window_closed(XfceNotifyWindow *window,
                           XfceNotifyCloseReason reason)
